# -*- coding:utf-8 -*-
from .AbstractApi import *

CORP_API_TYPE = {
    'GET_ACCESS_TOKEN': ['/cgi-bin/gettoken', 'GET'],
    'USER_CREATE': ['/cgi-bin/user/create?access_token=ACCESS_TOKEN', 'POST'],
    'USER_GET': ['/cgi-bin/user/get?access_token=ACCESS_TOKEN', 'GET'],
    'USER_UPDATE': ['/cgi-bin/user/update?access_token=ACCESS_TOKEN', 'POST'],
    'USER_DELETE': ['/cgi-bin/user/delete?access_token=ACCESS_TOKEN', 'GET'],
    'USER_BATCH_DELETE': ['/cgi-bin/user/batchdelete?access_token=ACCESS_TOKEN', 'POST'],
    'USER_SIMPLE_LIST': ['/cgi-bin/user/simplelist?access_token=ACCESS_TOKEN', 'GET'],
    'USER_LIST': ['/cgi-bin/user/list?access_token=ACCESS_TOKEN', 'GET'],
    'USERID_TO_OPENID': ['/cgi-bin/user/convert_to_openid?access_token=ACCESS_TOKEN', 'POST'],
    'OPENID_TO_USERID': ['/cgi-bin/user/convert_to_userid?access_token=ACCESS_TOKEN', 'POST'],
    'USER_AUTH_SUCCESS': ['/cgi-bin/user/authsucc?access_token=ACCESS_TOKEN', 'GET'],

    'DEPARTMENT_CREATE': ['/cgi-bin/department/create?access_token=ACCESS_TOKEN', 'POST'],
    'DEPARTMENT_UPDATE': ['/cgi-bin/department/update?access_token=ACCESS_TOKEN', 'POST'],
    'DEPARTMENT_DELETE': ['/cgi-bin/department/delete?access_token=ACCESS_TOKEN', 'GET'],
    'DEPARTMENT_LIST': ['/cgi-bin/department/list?access_token=ACCESS_TOKEN', 'GET'],

    'TAG_CREATE': ['/cgi-bin/tag/create?access_token=ACCESS_TOKEN', 'POST'],
    'TAG_UPDATE': ['/cgi-bin/tag/update?access_token=ACCESS_TOKEN', 'POST'],
    'TAG_DELETE': ['/cgi-bin/tag/delete?access_token=ACCESS_TOKEN', 'GET'],
    'TAG_GET_USER': ['/cgi-bin/tag/get?access_token=ACCESS_TOKEN', 'GET'],
    'TAG_ADD_USER': ['/cgi-bin/tag/addtagusers?access_token=ACCESS_TOKEN', 'POST'],
    'TAG_DELETE_USER': ['/cgi-bin/tag/deltagusers?access_token=ACCESS_TOKEN', 'POST'],
    'TAG_GET_LIST': ['/cgi-bin/tag/list?access_token=ACCESS_TOKEN', 'GET'],

    'BATCH_JOB_GET_RESULT': ['/cgi-bin/batch/getresult?access_token=ACCESS_TOKEN', 'GET'],

    'BATCH_INVITE': ['/cgi-bin/batch/invite?access_token=ACCESS_TOKEN', 'POST'],

    'AGENT_GET': ['/cgi-bin/agent/get?access_token=ACCESS_TOKEN', 'GET'],
    'AGENT_SET': ['/cgi-bin/agent/set?access_token=ACCESS_TOKEN', 'POST'],
    'AGENT_GET_LIST': ['/cgi-bin/agent/list?access_token=ACCESS_TOKEN', 'GET'],

    'MENU_CREATE': ['/cgi-bin/menu/create?access_token=ACCESS_TOKEN', 'POST'],
    'MENU_GET': ['/cgi-bin/menu/get?access_token=ACCESS_TOKEN', 'GET'],
    'MENU_DELETE': ['/cgi-bin/menu/delete?access_token=ACCESS_TOKEN', 'GET'],

    'MESSAGE_SEND': ['/cgi-bin/message/send?access_token=ACCESS_TOKEN', 'POST'],
    'MESSAGE_REVOKE': ['/cgi-bin/message/revoke?access_token=ACCESS_TOKEN', 'POST'],

    'MEDIA_GET': ['/cgi-bin/media/get?access_token=ACCESS_TOKEN', 'GET'],

    'GET_USER_INFO_BY_CODE': ['/cgi-bin/user/getuserinfo?access_token=ACCESS_TOKEN', 'GET'],
    'GET_USER_DETAIL': ['/cgi-bin/user/getuserdetail?access_token=ACCESS_TOKEN', 'POST'],

    'GET_TICKET': ['/cgi-bin/ticket/get?access_token=ACCESS_TOKEN', 'GET'],
    'GET_JSAPI_TICKET': ['/cgi-bin/get_jsapi_ticket?access_token=ACCESS_TOKEN', 'GET'],

    'GET_CHECKIN_OPTION': ['/cgi-bin/checkin/getcheckinoption?access_token=ACCESS_TOKEN', 'POST'],
    'GET_CHECKIN_DATA': ['/cgi-bin/checkin/getcheckindata?access_token=ACCESS_TOKEN', 'POST'],
    'GET_APPROVAL_DATA': ['/cgi-bin/corp/getapprovaldata?access_token=ACCESS_TOKEN', 'POST'],

    'GET_INVOICE_INFO': ['/cgi-bin/card/invoice/reimburse/getinvoiceinfo?access_token=ACCESS_TOKEN', 'POST'],
    'UPDATE_INVOICE_STATUS':
        ['/cgi-bin/card/invoice/reimburse/updateinvoicestatus?access_token=ACCESS_TOKEN', 'POST'],
    'BATCH_UPDATE_INVOICE_STATUS':
        ['/cgi-bin/card/invoice/reimburse/updatestatusbatch?access_token=ACCESS_TOKEN', 'POST'],
    'BATCH_GET_INVOICE_INFO':
        ['/cgi-bin/card/invoice/reimburse/getinvoiceinfobatch?access_token=ACCESS_TOKEN', 'POST'],

    'APP_CHAT_CREATE': ['/cgi-bin/appchat/create?access_token=ACCESS_TOKEN', 'POST'],
    'APP_CHAT_GET': ['/cgi-bin/appchat/get?access_token=ACCESS_TOKEN', 'GET'],
    'APP_CHAT_UPDATE': ['/cgi-bin/appchat/update?access_token=ACCESS_TOKEN', 'POST'],
    'APP_CHAT_SEND': ['/cgi-bin/appchat/send?access_token=ACCESS_TOKEN', 'POST'],

    'MINIPROGRAM_CODE_TO_SESSION_KEY': ['/cgi-bin/miniprogram/jscode2session?access_token=ACCESS_TOKEN', 'GET'],
}


class CorpApi(AbstractApi):
    def __init__(self, corpid, secert):
        super().__init__()
        self.corpid = corpid
        self.secert = secert
        self.access_toker = None

    def getAccessToken(self):
        if self.access_toker is None:
            self.refreshAccessToken()
        return self.access_toker

    def refreshAccessToken(self):
        response = self.httpCall(
            CORP_API_TYPE['GET_ACCESS_TOKEN'],
            {
                'corpid': self.corpid,
                'corpsecret': self.secert
            }
        )
        self.access_toker = response.get('access_token')



# -*- coding: utf-8 -*-
# Desc: 封装发送邮件模块
#
import email.utils
from smtplib import SMTP_SSL, SMTP, SMTPException
from email.mime.text import MIMEText
from email.mime.image import MIMEImage
from email.mime.multipart import MIMEMultipart
from email.header import Header
from email.utils import formataddr
import random
import os


class EmailTools(object):
    def __init__(self, subject, user, password, smtp,to_list, cc_list, port):
        self.Subject = subject
        self.USER = user
        self.FROM = ''
        self.PW = password
        self.SMTP = smtp
        self.PORT = port
        self.TO_LIST = to_list
        self.CC_LIST = cc_list
        self.init()

    def init(self):
        self.str = ""
        self.images = {}
        self.attr = []
        self.init_static()

    def init_static(self):
        self.str = """<html>
        <head>
        <style type="text/css">
            table.table-otl
        {
            border-collapse: collapse;
            margin: 0 auto;
            text-align: center;
        }
        table.table-otl  td, table.table-otl  th
        {
            border: 1px solid #cad9ea;
            color: #666666;
            height: 30px;
        }
        table.table-otl  thead th
        {
            background-color: #CCE8EB;
            width: 100px;
        }
        table.table-otl  tr:nth-child(odd)
        {
            background: #ffffff;
        }
        table.table-otl  tr:nth-child(even)
        {
            background: #F5FAFA;
        }
    </style></head>
    <body>
        """


    def add_str(self, s):
        """
        添加文本
        :param s:
        :return:
        """
        if not isinstance(s, str): raise TypeError ("s must be str or int, but gave {}".format(type(s)))
        self.str += ''' {} <br>'''.format(s)

    def add_image(self, image):
        """
        添加图片
        :param image:
        :return:
        """
        _cid = "image{}".format(random.randint(0, 999))
        self.add_str("""<img src="cid:{}">""".format(_cid))
        self.images[_cid] = image

    def add_attr(self, attr):
        """
        添加附件
        :param attr:
        :return:
        """
        self.attr.append(attr)

    def add_table(self, data, **kwargs):
        """
        添加表格
        :param: data 表格数据 格式：[[1, 2, 3, 4], [a, b, c, d]]
        :param: title 表格标题  非必须
        :return:
        """
        title = kwargs['title'] if 'title' in kwargs.keys() else None

        msg = """<table class="table-otl">"""
        if not isinstance(data, list): raise TypeError ("data must be [ list ], give {}".format(type(data)))
        if title:
            msg += """<caption> <h2> {} </h2></caption>""".format(title)

        head = data.pop(0)
        if head:
            msg += """<thead><tr>"""
            for h in head:
                msg += """<th>{}</th>""".format(h)
            msg += """</tr></thead>"""

        for i in data:
            msg += """<tr>"""
            for d in i:
                if isinstance(d, dict):
                    msg += """<td {}>{}</td>""".format(
                        ' '.join(str(x) + '="' + str(y) + '"' for x, y in d.items() if x != 'value'), d['value'])
                else:
                    msg += """<td>{}</td>""".format(d)
            msg += """</tr>"""
        msg += """</table>"""
        self.add_str(msg)

    def add_strong_str(self, s):
        if not isinstance(s, str): raise TypeError ("s must be str or int, but give {}".format(type(s)))
        self.str += '''<strong> {} </strong><br>'''.format(s)

    def _set_from(self, _from):
        self.FROM = _from

    def __get_str(self):
        return self.str

    def __get_image(self, cid, image):
        with open(image, 'rb') as fp:
            i = MIMEImage(fp.read(), _subtype='octet-stream')
            i.add_header('Content-ID', cid)
        return i

    def __get_file(self, file):
        with open(file, 'rb') as fp:
            i = MIMEText(fp.read(), 'base64', 'utf-8')
            i.add_header('Content-Type', 'application/octet-stream')
            i.add_header('Content-Disposition', 'attachment; filename=' + os.path.basename(file))

        return i

    def send_email(self):
        msgAlternative = MIMEMultipart()
        msgAlternative['From'] = self.USER
        msgAlternative['Subject'] = Header(self.Subject)
        msgAlternative['To'] = ",".join(self.TO_LIST)
        msgAlternative['Message-id'] = email.utils.make_msgid()
        if self.CC_LIST:
            msgAlternative['Cc'] = Header(",".join(self.CC_LIST))

        # 设定纯文本信息
        msgHtml = MIMEText(self.__get_str(), 'html', 'utf-8')
        msgAlternative.attach(msgHtml)

        # 设定内置图片信息
        if self.images:
            for cid in self.images.keys():
                i = self.__get_image(cid, self.images[cid])
                msgAlternative.attach(i)

        # 设定附件
        if self.attr:
            for file in self.attr:
                i = self.__get_file(file)
                msgAlternative.attach(i)
        if self.PORT == 25:
            try:
                smtp_obj = SMTP(self.SMTP, self.PORT)
                smtp_obj.connect(self.SMTP, self.PORT)
                smtp_obj.login(self.USER, self.PW)
                smtp_obj.sendmail(self.USER, self.TO_LIST + self.CC_LIST, msgAlternative.as_string())
                smtp_obj.close()
                return True, "邮件发送成功"
            except SMTPException as e:
                return False, "邮件发送失败: {}".format(e)
        elif self.PORT == 465:
            smtp_obj = SMTP_SSL(self.SMTP, self.PORT)
            try:
                smtp_obj.login(self.USER, self.PW)
                smtp_obj.sendmail(self.USER, self.TO_LIST + self.CC_LIST, msgAlternative.as_string())
                return True, "邮件发送成功"
            except SMTPException as e:
                return False, "邮件发送失败: {}".format(e)
            finally:
                smtp_obj.quit()


class EmailApi(object):
    def __init__(self, config):
        self.config = config
        if not isinstance(self.config, dict):
            raise Exception('config type Error: {}, need [dict]'.format(type(self.config)))
        self.apps = {}
        self.init_app()

    def init_app(self):
        for k in self.config.keys():
            self.apps[k] = EmailTools(
                self.config[k]['name'],
                self.config[k]['user'],
                self.config[k]['password'],
                self.config[k]['smtp'],
                self.config[k]['to_list'],
                self.config[k]['cc_list'],
                self.config[k]['port'],
            )

    def __getattr__(self, item) -> EmailTools:
        return self.apps.get(item)





# -*- coding: utf-8 -*-

import random
from .src.CoreApi import CorpApi, CORP_API_TYPE


class WechatApp(object):
    def __init__(self, corp_id, app_id, app_secret):
        self.available_msg_type = ['text', 'markdown']
        self.corp_id = corp_id
        self.app_id = app_id
        self.app_secret = app_secret
        self.api = CorpApi(corp_id, app_secret)

        self.app_info = {}
        self.allow_parts = {}
        self.allow_users = {}

        self.get_app_info()
        self.get_departs_info()
        self.get_users_info()

    def send(self, msg_type, to_users_list=None, to_group_list=None, msg_string=''):
        """

        :param msg_type: 消息类型：text or markdown
        :param to_users_list: 接收用户列表
        :param to_group_list: 接收组列表
        :param msg_string: message
        :return:
        """
        if to_group_list is None:
            to_group_list = []
        if to_users_list is None:
            to_users_list = []
        if msg_type not in self.available_msg_type:
            return False, ('msg_type_error:{},available:{}'.format(msg_type, ",".join([str(k) for k in self.available_msg_type])))

        for user in to_users_list:
            user = str(user)
            if user not in self.allow_users:
                return False, ('to_users_list_error: {}, available: {}'.format(user, ','.join([str(k) for k in self.allow_users.keys()])))

        for part in to_group_list:
            part = str(part)
            if part not in self.allow_parts:
                return False, ('to_group_list_error: {}, available: {}'.format(part, ','.join([str(k) for k in self.allow_parts.keys()])))

        to_users_list = [str(self.allow_users[str(user)]) for user in to_users_list]
        to_group_list = [str(self.allow_parts[str(group)]) for group in to_group_list]
        if msg_type == 'text':
            return True, self.send_text(to_users_list=to_users_list, to_group_list=to_group_list, msg_string=msg_string)
        elif msg_type == 'markdown':
            return True, self.send_markdown(to_users_list=to_users_list, to_group_list=to_group_list, markdown=msg_string)

    def get_app_info(self):
        response = self.api.httpCall(
            CORP_API_TYPE['AGENT_GET'],
            {
                'agentid': self.app_id,
            }
        )
        self.app_info = response

    def get_departs_info(self):
        for pid in self.app_info.get('allow_partys', {}).get('partyid', []):
            response = self.api.httpCall(
                CORP_API_TYPE['DEPARTMENT_LIST'],
                {
                    "id": str(pid)
                }
            )
            for k in response['department']:
                self.allow_parts[str(k['name'])] = int(k['id'])

    def get_users_info(self):
        for k, pid in self.allow_parts.items():
            response = self.api.httpCall(
                CORP_API_TYPE['USER_SIMPLE_LIST'],
                {
                    "department_id": str(pid),
                    "fetch_child": "1",
                }
            )
            for user in response['userlist']:
                self.allow_users[str(user['name'])] = user['userid']
        for user in self.app_info['allow_userinfos'].get('user', []):
            response = self.api.httpCall(
                CORP_API_TYPE['USER_GET'],
                {
                    "userid": str(user['userid'])
                }
            )
            name = response.get('name')
            self.allow_users[str(name)] = user['userid']

    def send_text(self, to_users_list=[], to_group_list=[], msg_string=""):
        response = self.api.httpCall(
            CORP_API_TYPE['MESSAGE_SEND'],
            {
                "touser": "|".join(to_users_list),
                "toparty": "|".join(to_group_list),
                "agentid": str(self.app_id),
                'msgtype': 'text',
                'climsgid': 'climsgidclimsgid_%f' % (random.random()),
                'text': {
                    'content': msg_string,
                },
                'safe': 0,
            }
        )

        return response

    def send_markdown(self, to_users_list=None, to_group_list=None, markdown=""):
        if to_group_list is None:
            to_group_list = []
        if to_users_list is None:
            to_users_list = []
        response = self.api.httpCall(
            CORP_API_TYPE['MESSAGE_SEND'],
            {
                "touser": "|".join(to_users_list),
                "toparty": "|".join(to_group_list),
                "agentid": str(self.app_id),
                'msgtype': 'markdown',
                'climsgid': 'climsgidclimsgid_%f' % (random.random()),
                "markdown": {
                    "content": str(markdown)
                },
                'safe': 0,
            }
        )
        return response


class WechatApi(object):
    def __init__(self, config):
        self.config = config
        self.NAME = config['NAME']
        self.CORP_ID = config['CORP_ID']
        self.APP_LIST_CONFIG = config['APP_LIST']
        self.apps = {}
        self.trans = {}
        self.init_apps()

    def init_apps(self):
        for k, v in self.APP_LIST_CONFIG.items():
            if v.get('switch') != 'on': continue
            app = WechatApp(self.CORP_ID, v['APP_ID'], v['APP_SECRET'])
            self.apps[k] = app



# -*- coding: utf-8 -*-
import requests
import json

DEBUG = False


class ApiException(Exception):
    def __init__(self, errCode, errMsg):
        self.errCode = errCode
        self.errMsg = errMsg


class AbstractApi(object):
    def __init__(self):
        return

    def getAccessToken(self):
        raise NotImplementedError

    def refreshAccessToken(self):
        raise NotImplementedError

    def getSuiteAccessToken(self):
        raise NotImplementedError

    def refreshSuiteAccessToken(self):
        raise NotImplementedError

    def getProviderAccessToken(self):
        raise NotImplementedError

    def refreshProviderAccessToken(self):
        raise NotImplementedError

    @staticmethod
    def __makeUrl(shortUrl):
        base = "https://qyapi.weixin.qq.com"
        if shortUrl[0] == '/':
            return base + shortUrl
        else:
            return base + '/' + shortUrl

    def __appendToken(self, url):
        if 'SUITE_ACCESS_TOKEN' in url:
            return url.replace('SUITE_ACCESS_TOKEN', self.getSuiteAccessToken())
        elif 'PROVIDER_ACCESS_TOKEN' in url:
            return url.replace('PROVIDER_ACCESS_TOKEN', self.getProviderAccessToken())
        elif 'ACCESS_TOKEN' in url:
            return url.replace('ACCESS_TOKEN', self.getAccessToken())
        else:
            return url

    def __httpPost(self, url, args):
        realUrl = self.__appendToken(url)
        if DEBUG is True:
            print(realUrl)
        return requests.post(realUrl, data=json.dumps(args, ensure_ascii=False).encode('utf-8')).json()

    def __httpGet(self, url):
        realUrl = self.__appendToken(url)

        if DEBUG is True:
            print(realUrl)
        return requests.get(realUrl).json()

    def __post_file(self, url, media_file):
        return requests.post(url, file=media_file).json()

    @staticmethod
    def __tokenExpired(errCode):
        if errCode == 40014 or errCode == 42001 or errCode == 42007 or errCode == 42009:
            return True
        else:
            return False

    def __refreshToken(self, url):
        if 'SUITE_ACCESS_TOKEN' in url:
            self.refreshSuiteAccessToken()
        elif 'PROVIDER_ACCESS_TOKEN' in url:
            self.refreshProviderAccessToken()
        elif 'ACCESS_TOKEN' in url:
            self.refreshAccessToken()

    @staticmethod
    def __checkResponse(response):
        errCode = response.get('errcode')
        errMsg = response.get('errmsg')

        if errCode is 0:
            return response
        else:
            print(errCode, errMsg)
            raise ApiException(errCode, errMsg)

    @staticmethod
    def __appendArgs(url, args):
        if args is None:
            return url

        for key, value in args.items():
            if '?' in url:
                url += ('&' + key + '=' + value)
            else:
                url += ('?' + key + '=' + value)
        return url

    def httpCall(self, urlType, args=None):
        shortUrl = urlType[0]
        method = urlType[1]
        response = {}
        for retryCnt in range(0, 3):
            if 'POST' == method:
                url = self.__makeUrl(shortUrl)
                response = self.__httpPost(url, args)
            elif 'GET' == method:
                url = self.__makeUrl(shortUrl)
                url = self.__appendArgs(url, args)
                response = self.__httpGet(url)
            else:
                raise ApiException(-1, "unknown method type")

            if self.__tokenExpired(response.get('errcode')):
                self.__refreshToken(shortUrl)
                retryCnt += 1
                continue
            else:
                break

        return self.__checkResponse(response)



# -*- coding: utf-8 -*-
import re
import urllib.parse
import time
import requests
import hmac
import json
import base64
import hashlib


class DingTalkException(IOError):
    def __init__(self, *args):
        super().__init__(*args)


class DingTalkApp:
    def __init__(self, dingtalk_token, webhook_secret=None):
        _signs = self._secure_sign(webhook_secret)
        self.msg_type = ['text', 'markdown']
        _token = dingtalk_token
        self.robot_url = "https://oapi.dingtalk.com/robot/send?access_token={}&timestamp={}&sign={}".format(
            _token, _signs[0], _signs[1]
        )

    @staticmethod
    def _secure_sign(secret):
        """
        加签,只有机器人选择加签安全验证方式时使用
        :param secret: secret for DingDing robot!
        :return: tuple(timestamp, sign)
        """
        timestamp = str(round(time.time() * 1000))
        _secret = secret
        secret_enc = _secret.encode('utf-8')
        string_to_sign = '{}\n{}'.format(timestamp, _secret)
        string_to_sign_enc = string_to_sign.encode('utf-8')
        hmac_code = hmac.new(secret_enc, string_to_sign_enc, digestmod=hashlib.sha256).digest()
        sign = urllib.parse.quote_plus(base64.b64encode(hmac_code))
        return timestamp, sign

    def send(self, title=None, content=None, msg_type='text', at_users=None) -> tuple:
        """
        发送消息
        :param title:
        :param content:
        :param msg_type:
        :param at_users:
        :return: tuple(Bool,str)
        """
        if title is None:
            title = ""
        if content is None:
            content = ""
        if msg_type == 'text':
            contents = {'msgtype': msg_type,msg_type: {'title': title, 'content': content}}
        elif msg_type == 'markdown':
            contents = {'msgtype': msg_type, msg_type: {'title': title, 'text': content}}
        else:
            raise DingTalkException("不支持的消息类型：{}, 当前仅支持：{}".format(msg_type, ','.join(x for x in self.msg_type)))

        contents['at'] = self._load_at_list(at_users)
        headers = {'Content-Type': 'application/json'}
        resp = requests.post(self.robot_url, headers=headers, data=json.dumps(contents))
        if resp.status_code == 200:
            if resp.json().get('errcode') != 0:
                raise DingTalkException(resp.json().get('errmsg'))
            else:
                return True, "DingDing notify send succeed! "
        return False, "DingDing notify send failed! "

    @staticmethod
    def _load_at_list(users=None):
        if users is None:
            return []
        at_users = {}
        if users == 'all' or users == 'All':
            at_users['isAtAll'] = 'true'
        else:
            for user in users.split(','):
                if re.findall(r"1[3-9]\d{9}$", user):
                    if 'atMobiles' not in at_users.keys():
                        at_users['atMobiles'] = [user]
                    else:
                        at_users['atMobiles'].append(user)
                else:
                    if 'atUserIds' not in at_users.keys():
                        at_users['atUserIds'] = [user]
                    else:
                        at_users['atUserIds'].append(user)
        return at_users


class DingTalkApi(object):
    def __init__(self, config):
        self.config = config
        self.NAME = config['NAME']
        self.APP_LIST_CONFIG = config['APP_LIST']
        self.apps = {}
        self.init_apps()

    def init_apps(self):
        for k, v in self.APP_LIST_CONFIG.items():
            app = DingTalkApp(v['token'], v['secret'])
            self.apps[k] = app

    def __getattr__(self, item):
        return self.apps.get(item)



# -*- coding: utf-8 -*-
from setuptools import find_packages, setup
from otl.__version__ import __title__, __version__, __author__, __license__

requires = []
with open('requirements.txt', 'r') as f1:
    for line in f1.readlines():
        requires.append(line.strip())

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()

setup(
    name=__title__,
    version=__version__,
    description="运维工具集",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author=__author__,
    author_email='xuyong@qq.com',
    url='https://github.com/xuyonggit/otl.git',
    packages=find_packages(),
    install_requires=requires,
    license=__license__,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ]
)



class Sql(object):
    def valid(self, sql):
        # select 语句必须包含条件where
        if sql.startswith("SELECT") or sql.startswith("select"):
            if 'where' not in sql and 'WHERE' not in sql:
                return "select 语句必须包含条件where", ""
        # delete 语句必须包含条件where
        elif sql.startswith("DELETE") or sql.startswith("delete"):
            if 'where' not in sql and 'WHERE' not in sql:
                return "delete 语句必须包含条件where", ""
        # # sql 语句中尽量避免in操作
        # elif " in " in sql:
        #     return "sql中不允许使用in操作", ""
        return None, sql


valid_sql = Sql()



# -*- coding: utf-8 -*-
import pymysql
from dbutils.pooled_db import PooledDB
from otl.db.common.SQL import valid_sql


class MysqlDbBase:
    host = None
    port = 3306
    user = None
    password = None
    db = None
    __pool = None

    def __init__(self):
        self.pool = self.__get_conn_pool()

    def __get_conn_pool(self):
        if not self.__pool:
            try:
                self.__pool = PooledDB(
                    creator=pymysql,
                    host=self.host,
                    port=self.port,
                    user=self.user,
                    password=self.password,
                    db=self.db
                )
            except Exception as err:
                raise Exception("Error: connect mysql error %s", err)
        return self.__pool

    def _get_connection(self):
        conn = self.pool.connection()
        cursor = conn.cursor(pymysql.cursors.DictCursor)
        return conn, cursor

    def _close_connection(self, conn, cursor):
        if conn:
            conn.close()
        if cursor:
            cursor.close()

    def queryone(self, sql, params=()) -> dict:
        """
        单条数据查询
        :param sql:
        :param params:
        :return: result dict
        """
        conn, cur = self._get_connection()
        try:
            err, v_sql = valid_sql.valid(sql)
            if err:
                raise Exception("sql valid Error: %s", err)
            row = cur.execute(v_sql, params)
            if row == 0:
                return {}
            else:
                return cur.fetchone()
        except Exception as err:
            raise Exception(f"Query Error: {err} = {sql}")
        finally:
            self._close_connection(conn, cur)

    def query(self, sql, params=()) -> list:
        """
        查询方法
        :param params:
        :param sql:
        :return: result List
        """
        conn, cur = self._get_connection()
        try:
            result = []
            # 校验sql合规
            err, v_sql = valid_sql.valid(sql)
            if err is not None:
                raise Exception(err)

            row = cur.execute(v_sql, params)
            if row == 0:
                return result
            else:
                for rest in cur.fetchall():
                    result.append(rest)
                return result
        except Exception as err:
            raise Exception(f"Query Error: {err} - {sql}")
        finally:
            self._close_connection(conn, cur)

    def insert(self, sql, params=()) -> int:
        """
        插入方法
        :param params:
        :param sql:
        :return:
        """
        return self._update("Insert", sql, params)

    def update(self, sql, params=()) -> int:
        """
        更新方法
        :param params:
        :param sql:
        :return:
        """
        return self._update("Update", sql, params)

    def delete(self, sql, params=()) -> int:
        """
        删除方法
        :param params:
        :param sql:
        :return:
        """
        return self._update("Delete", sql, params)

    def _update(self, modify_type, sql, params=()) -> int:
        conn, cur = self._get_connection()
        try:
            # 校验sql合规
            err, v_sql = valid_sql.valid(sql)
            if err is not None:
                raise Exception(err)
            rows = cur.execute(v_sql, params)
            conn.commit()
            return rows
        except Exception as err:
            raise Exception(f"{modify_type} Error: {err} - {sql}")
        finally:
            self._close_connection(conn, cur)
